#from borb.pdf.canvas.layout.page_layout.page_layout import PageLayout as BorbPageLayout
#from borb.pdf.canvas.layout.page_layout.multi_column_layout import SingleColumnLayout as BorbSingleColumnLayout
#from borb.pdf import PageLayout as BorbPageLayout
from borb.pdf import SingleColumnLayout as BorbSingleColumnLayout
#from borb.pdf import MultiColumnLayout as BorbMultiColumnLayout


class SingleColumnLayout(BorbSingleColumnLayout):
    pass


